#
# Copyright 2008 TKK/ComNet
# Released under GPLv3. See LICENSE.txt for details.
#

require File.dirname(__FILE__) + '/spec_helper'

include Dtn

describe Bundle, "when first created" do
  
  before(:all) do
    @payload_types = [
      DTN_PAYLOAD_MEM,
      DTN_PAYLOAD_FILE,
      DTN_PAYLOAD_TEMP_FILE
    ]
    @bundles = {
      :mem_bundle => Bundle.new,
      :file_bundle => Bundle.new("", DTN_PAYLOAD_FILE),
      :tempfile_bundle => Bundle.new("", DTN_PAYLOAD_TEMP_FILE)
    }
  end
  
  it "should have a valid payload type (location)" do
    @bundles.each_value do |b|
      @payload_types.should include(b.payload.location)
    end
  end
  
  it "should have buffer length of 0" do
    @bundles.each_value do |b|
      b.payload.buf.buf_len.should == 0
    end
  end
  
  it "should not have a source eid" do
    @bundles.each_value do |b|
      b.spec.source.uri.should be_empty
    end
  end
  
  it "should have an accessible, empty payload if payload type is MEM" do
    @bundles[:mem_bundle].payload.data.should be_empty
  end
  
  it "should have payload file name if payload type is FILE or TEMP_FILE" do
    @bundles[:file_bundle].payload.file.should be_an_instance_of(String)
    @bundles[:tempfile_bundle].payload.file.should be_an_instance_of(String)
  end
  
end
