#
# Copyright 2008 TKK/ComNet
# Released under GPLv3. See LICENSE.txt for details.
#

# This file contains Rspec tests for the DTN client. Note that running
# these test requires that an instance of dtnd is running. This means that
# in addition to testing our code, we are also testing interaction with a
# real bundle router (keep this in mind when tests fail).

# TODO: mock it?

require File.dirname(__FILE__) + '/spec_helper'

include Dtn

describe Client, "when first created" do
  
  before(:all) do
    eid = "dtn://lpmbp.dtn/rspec"
    @client = Client.new(eid)
  end
  
  it "should have a valid endpoint id" do
    @client.uri.should be_an_instance_of(String)
    @client.uri.should =~ /dtn:\/\//
  end
  
  it "should be connected and registered" do
    @client.should be_connected
    @client.regid.should >= 0
  end
  
  it "should unregister and close gracefully" do
    lambda { @client.unregister }.should_not raise_error
    lambda { @client.close }.should_not raise_error
  end  
end

describe Client, "when interacting with dtnd" do
  
  before(:each) do
    eid = "dtn://lpmbp.dtn/rspec"
    @client = Client.new(eid)
  end
  
  it "should be able to succesfully send a bundle to itself" do
    sent_data = "TESTING 123 --- 000"    
    bundle = Bundle.new(@client.uri)    
    bundle.payload.data = sent_data
    @client.send_bundle(bundle).should_not be_nil
    received_bundle = @client.receive_bundle(1000)
    received_bundle.should_not be_nil
    received_bundle.payload.location.should == bundle.payload.location
    received_bundle.payload.data.should == sent_data
  end
  
  it "should raise Dtn::Timeout if there is no bundle to receive" do
    lambda { @client.receive_bundle(100) }.should raise_error(Dtn::Timeout)
  end
  
  after(:each) do
    @client.unregister
    @client.close
  end
end
