#
# Copyright 2008 TKK/ComNet
# Released under GPLv3. See LICENSE.txt for details.
#

require File.dirname(__FILE__) + '/spec_helper'
require 'mhtml'

include Mhtml

def eid; 'http://lpmbp.dtn/' end

def create_multipart_message
  message = Message.new(eid)
  message.add_file_part(__FILE__, 'self.file')
  message.add_file_part(File.dirname(__FILE__) + '/test_webroot/test.jpg', 'test.jpg')
  message.add_file_part(File.dirname(__FILE__) + '/test_webroot/test.jpg', 'test.jpg', {}, true)
  message.add_file_part(File.dirname(__FILE__) + '/test_webroot/test.html', 'test.html', {}, true)
  message.to_s # Serialization adds the boundary (don't return this though)
  message # Return the message object
end  


describe Message, "when first created" do
  
  before(:each) do
    @message = Message.new(eid)
  end  
  
  it "should contain the default headers" do
    @message.header['Mime-Version'].should == '1.0'
    @message.header['Content-Type'].should =~ /multipart\/related/
  end  
end


describe Message, "when containing more than one part" do
  
  # Create a multipart message with a text part and a base64 encoded file part
  before(:each) do
    @message = create_multipart_message
  end    
  
  it "should be a multipart message" do
    @message.should be_multipart
  end
  
  it "should have a boundary specified in the Content-Type header" do
    @message.header['Content-Type'].should =~ /boundary=/
  end
  
  it "should have an absolute Content-Location in the top-level part" do
    @message.header['Content-Location'].should =~ /http:\/\/(.)*/
  end
  
  it "should have a Content-Location for all parts" do
    @message.each_part do |part|
      part.header['Content-Location'].should_not be_nil
    end
  end
    
  it "should perform encoding on binary files when asked to do so" do
    @message.part(2).header['Content-Transfer-Encoding'].should == 'base64'
    @message.part(3).header['Content-Transfer-Encoding'].should == 'quoted-printable'
  end
end


describe Message, "when iterationg through parts" do
  
  before(:each) do
    @message = create_multipart_message
  end      
  
  # it "should extract the content location" do
  #   i = 0
  #   @message.each_file_part do |location, data|
  #     location.should == File.join(@message.header['Content-Location'], @message.part(i).header['Content-Location'])
  #     i += 1
  #   end
  # end
  
  it "should perform decoding for encoded parts" do
    i = 0
    @message.each_file_part do |location, data|
      if @message.part(i).header['Content-Transfer-Encoding'] == 'base64'
        data.should == RMail::Utils.base64_decode(@message.part(i).body)
      end
      if @message.part(i).header['Content-Transfer-Encoding'] == 'quoted-printable'
        data.should == RMail::Utils.quoted_printable_decode(@message.part(i).body)
      end      
      i += 1
    end    
  end  
end


describe Message, "when handled as a string of MHTML" do
  
  # Create a multipart message with a text part and a base64 encoded file part
  before(:each) do
    @mhtml = Message.new(eid).to_s
  end    
  
  it "should be parseable into an Mhtml::Message object" do
    m = Mhtml::Message.parse(@mhtml)
    m.should be_an_instance_of(Mhtml::Message)
  end    
end


